<?php

/**
 * @file
 * Form builder, submit, validate and operation functions for managing
 * complaints about comments.
 */

/**
 * Menu callback; administration of listing complaints on comments.
 */
function comment_abuse_admin() {
  $edit = $_POST;
  if (isset($edit['operation']) && ($edit['operation'] == 'delete') && isset($edit['comments']) && $edit['comments']) {
    module_load_include('inc', 'comment', 'comment.admin');
    return drupal_get_form('comment_abuse_multiple_delete_confirm');
  }
  else {
    return drupal_get_form('comment_abuse_list_form');
  }
}

/**
 * Form builder. Builds the complaints on comments overview form for the admin.
 */
function comment_abuse_list_form() {
  // Build an 'Update options' form.
  $form['options'] = array(
    '#type' => 'fieldset', '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">', '#suffix' => '</div>',
  );

  $options = array();
  $options['publish'] = t('Publish the selected comments');
  $options['delete'] = t('Delete the selected comments');
  $options['unpublish'] = t('Unpublish the selected comments');
  $options['remove_complaint'] = t('Remove all complaints for selected comments');

  $form['options']['operation'] = array('#type' => 'select', '#options' => $options, '#default_value' => 'publish');
  $form['options']['submit'] = array('#type' => 'submit', '#value' => t('Update'));

  // Load the comments that we want to display.
  $form['header'] = array(
    '#type' => 'value', '#value' => array(
      theme('table_select_header_cell'),
      array('data' => t('Comment ID'), 'field' => 'cid'),
      array('data' => t('Status'), 'field' => 'status'),
      array('data' => t('Complaints'), 'field' => 'status'),
      array('data' => t('Subject'), 'field' => 'subject'),
      array('data' => t('Author'), 'field' => 'name'),
      array('data' => t('Posted in'), 'field' => 'node_title'),
      array('data' => t('Time'), 'field' => 'timestamp', 'sort' => 'desc'),
      array('data' => t('Operations')),
    ),
  );

  // Limiting results on page.
  if (isset ($_GET['page-limit']) && ((int) ($_GET['page-limit']) != 0 || $_GET['page-limit'] == 'ALL')) {
    $page_limit = $_GET['page-limit'];
  }
  else {
    $page_limit = __COMMENT_ABUSE_DEFAULT_COMPLAINTS_ON_PAGE;
  }


  $query = 'SELECT c.subject, c.nid, c.cid, c.comment, ca.timestamp, c.status, c.name, c.homepage, u.name AS registered_name,
      u.uid, n.title AS node_title, COUNT(ca.cid) AS complaint_count FROM {comments} c
      INNER JOIN {users} u ON u.uid = c.uid
      INNER JOIN {node} n ON n.nid = c.nid
      INNER JOIN {comment_abuse} ca ON c.cid = ca.cid
      GROUP BY ca.cid' . tablesort_sql($form['header']['#value']);

  $query .= ($page_limit != 'ALL') ? " LIMIT $page_limit" : '';
  $result = db_query($query);

  // Build a table listing the appropriate comments.
  $destination = drupal_get_destination();
  while ($comment = db_fetch_object($result)) {

    $comments[$comment->cid] = '';
    $comment->name = $comment->uid ? $comment->registered_name : $comment->name;

    if ($comment->status == 0) {
      $comment->status = t('published');
    }
    else {
      $comment->status = t('unpublished');
    }
    $form['cid'][$comment->cid] = array('#value' => $comment->cid);
    $form['status'][$comment->cid] = array('#value' => $comment->status);
    $form['complaint_count'][$comment->cid] = array('#value' => $comment->complaint_count);
    $form['subject'][$comment->cid] = array('#value' => l(truncate_utf8($comment->subject, __COMMENT_ABUSE_STRING_CROP, TRUE, TRUE), 'node/' . $comment->nid, array('attributes' => array('title' => truncate_utf8($comment->comment, 128)), 'fragment' => 'comment-' . $comment->cid)));
    $form['username'][$comment->cid] = array('#value' => theme('username', $comment));
    $form['node_title'][$comment->cid] = array('#value' => l(truncate_utf8($comment->node_title, __COMMENT_ABUSE_STRING_CROP, TRUE, TRUE), 'node/' . $comment->nid));
    $form['timestamp'][$comment->cid] = array('#value' => format_date($comment->timestamp, 'medium'));
    $form['operations'][$comment->cid] = array('#value' => l(t('edit'), 'comment/edit/' . $comment->cid, array('query' => $destination)));
  }

  $form['comments'] = array('#type' => 'checkboxes', '#options' => isset($comments) ? $comments: array());

  return $form;
}

/**
 * Theme the complaints on comments, admin form.
 */
function theme_comment_abuse_list_form($form) {
  $output = drupal_render($form['options']);
  if (isset($form['subject']) && is_array($form['subject'])) {
    foreach (element_children($form['subject']) as $key) {
      $row = array();
      $row[] = drupal_render($form['comments'][$key]);
      $row[] = drupal_render($form['cid'][$key]);
      $row[] = drupal_render($form['status'][$key]);
      $row[] = drupal_render($form['complaint_count'][$key]);
      $row[] = drupal_render($form['subject'][$key]);
      $row[] = drupal_render($form['username'][$key]);
      $row[] = drupal_render($form['node_title'][$key]);
      $row[] = drupal_render($form['timestamp'][$key]);
      $row[] = drupal_render($form['operations'][$key]);
      $rows[] = $row;
    }
  }
  else {
    $rows[] = array(array('data' => t('No complaints about the comments.'), 'colspan' => '6'));
  }
  $output .= theme('table', $form['header']['#value'], $rows);

  // Limit complaints on comments on page.
  global $base_url;
  $url = $base_url . '?';
  foreach ($_REQUEST as $key => $value) {
    $url .= ($key != 'q' && $key != 'page-limit') ? '&' : '';
    // Don't add old page-limit parameter.
    $url .= ($key != 'page-limit') ? "$key=$value" : '';
  }

  $output .= t('Limit complaints on page:');
  $output .= ' ' . l(10, $url . '&page-limit=10');
  $output .= ' ' . l(25, $url . '&page-limit=25');
  $output .= ' ' . l(50, $url . '&page-limit=50');
  $output .= ' ' . l(100, $url . '&page-limit=100');
  $output .= ' ' . l(t('Show all'), $url . '&page-limit=ALL');

  $output .= drupal_render($form);
  return $output;
}

/**
 * Validate comment_abuse_admin_overview form submissions.
 *
 * We can't execute any 'Update options' if no comments were selected.
 */
function comment_abuse_list_form_validate($form, &$form_state) {
  $form_state['values']['comments'] = array_diff($form_state['values']['comments'], array(0));
  if (count($form_state['values']['comments']) == 0) {
    form_set_error('', t('Please select one or more comments to perform the update on.'));
  }
}

/**
 * Process admin form for complaints on comments list.
 */
function comment_abuse_list_form_submit(&$form, &$form_state) {
  $operations = comment_operations();
  if (!empty($operations[$form_state['values']['operation']][1])) {
    // Extract the appropriate database query operation.
    $query = $operations[$form_state['values']['operation']][1];
    foreach ($form_state['values']['comments'] as $cid => $value) {
      if ($value) {
        // Perform the update action, then refresh node statistics.
        db_query($query, $cid);
        $comment = _comment_load($cid);
        _comment_update_node_statistics($comment->nid);
        // Allow modules to respond to the updating of a comment.
        comment_invoke_comment($comment, $form_state['values']['operation']);
        // Add an entry to the watchdog log.
        watchdog('content', 'Comment: updated %subject.', array('%subject' => $comment->subject), WATCHDOG_NOTICE, l(t('view'), 'node/' . $comment->nid, array('fragment' => 'comment-' . $comment->cid)));
      }
    }
    cache_clear_all();
    drupal_set_message(t('The update has been performed.'));
    $form_state['redirect'] = __COMMENT_ABUSE_LISTING_PATH;
  }
  // Remove complaints on comments.
  if ($form_state['values']['operation'] == 'remove_complaint') {
    $checked_rows = $form_state['values']['comments'];
    $r = db_query('DELETE FROM {comment_abuse} WHERE cid IN (' . db_placeholders($checked_rows) . ')', $checked_rows);
    $form_state['redirect'] = __COMMENT_ABUSE_LISTING_PATH;
  }
}

/**
 * List the selected comments and verify that the admin really wants to delete
 * them.
 * @see comment_abuse_multiple_delete_confirm_submit()
 */
function comment_abuse_multiple_delete_confirm(&$form_state) {
  $edit = $form_state['post'];

  $form['comments'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);
  // Function array_filter() returns only elements with actual values.
  $comment_counter = 0;
  foreach (array_filter($edit['comments']) as $cid => $value) {
    $comment = _comment_load($cid);
    if (is_object($comment) && is_numeric($comment->cid)) {
      $subject = db_result(db_query('SELECT subject FROM {comments} WHERE cid = %d', $cid));
      $form['comments'][$cid] = array('#type' => 'hidden', '#value' => $cid, '#prefix' => '<li>', '#suffix' => check_plain($subject) . '</li>');
      $comment_counter++;
    }
  }
  $form['operation'] = array('#type' => 'hidden', '#value' => 'delete');

  if (!$comment_counter) {
    drupal_set_message(t('There do not appear to be any comments to delete or your selected comment was deleted by another administrator.'));
    drupal_goto(__COMMENT_ABUSE_LISTING_PATH);
  }
  else {
    return confirm_form($form,
                        t('Are you sure you want to delete these comments and all their children?'),
                        __COMMENT_ABUSE_LISTING_PATH, t('This action cannot be undone.'),
                        t('Delete comments'), t('Cancel'));
  }
}

/**
 * Process comment_abuse_multiple_delete_confirm form submissions.
 *
 * Perform the actual comment deletion.
 */
function comment_abuse_multiple_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['comments'] as $cid => $value) {
      $comment = _comment_load($cid);
      _comment_delete_thread($comment);
      _comment_update_node_statistics($comment->nid);
    }
    cache_clear_all();
    drupal_set_message(t('The comments have been deleted.'));
  }
  $form_state['redirect'] = __COMMENT_ABUSE_LISTING_PATH;
}

/**
 * Form builder for comment abuse options.
 */
function comment_abuse_options_form() {
  $form['comment_abuse_complaint_received_message'] = array(
    '#type' => 'textfield',
    '#title' => t('Comment abuse complaint received message'),
    '#default_value' => variable_get(
      'comment_abuse_complaint_received_message',
      __COMMENT_ABUSE_COMPLAINT_RECEIVED_MESSAGE
    ),
    '#required' => TRUE,
    '#size' => 80,
  );

  $form['comment_abuse_complaint_not_received_message'] = array(
    '#type' => 'textfield',
    '#title' => t('Comment abuse complaint not received message'),
    '#default_value' => variable_get(
      'comment_abuse_complaint_not_received_message',
      __COMMENT_ABUSE_COMPLAINT_NOT_RECEIVED_MESSAGE
    ),
    '#required' => TRUE,
    '#size' => 80,
  );

  $form['comment_abuse_complaint_link_text'] = array(
    '#type' => 'textfield',
    '#title' => t('Complaint link text'),
    '#default_value' => variable_get(
      'comment_abuse_complaint_link_text',
      __COMMENT_ABUSE_COMPLAINT_LINK_TEXT
    ),
    '#required' => TRUE,
    '#size' => 80,
  );

  return system_settings_form($form);
}
